@@ -25,5 +25,8 @@ double fact_function_jr(double);
 int almost_integer(double, double);
 
 void error_occured(char *, int);
+void mess(char *, int);
+void messd(char *, double);
+void verify_malloc(void *);
 
 #endif /* AUX_HEADERS_H */
