@@ -65,8 +65,6 @@ void assign_accel(void)
 	accel_button(CODE_9, 0, GDK_KP_9);
 /*	accel_button(CODE_9, 0, GDK_KP_Prior); */
 
-/*  	accel_button(CODE_UNDO, 0, GDK_Clear); */
-
 	accel_button(CODE_ADD, 0, GDK_KP_Add);
 	accel_button(CODE_ADD, GDK_SHIFT_MASK, GDK_plus); 
 
@@ -91,7 +89,6 @@ void assign_accel(void)
 	accel_button(CODE_COS, 0, GDK_c);
 	accel_button(CODE_TAN, 0, GDK_t);
 
-
 	accel_button(CODE_EQ, 0, GDK_equal);
 
 	accel_button(CODE_DOT, 0, GDK_KP_Decimal);
@@ -99,13 +96,11 @@ void assign_accel(void)
 	accel_button(CODE_DOT, 0, GDK_comma);
 
 	accel_button(CODE_CLEAR_ALL, 0, GDK_Escape);
-  	accel_button(CODE_CLEAR_ALL, 0, GDK_Delete); 
 
 	accel_button(CODE_INFO, GDK_SHIFT_MASK, GDK_question);
-	accel_button(CODE_OPTIONS, GDK_SHIFT_MASK, GDK_exclam);
+	accel_button(CODE_SETTINGS, GDK_SHIFT_MASK, GDK_exclam);
 
 	accel_button(CODE_UNDO, GDK_CONTROL_MASK, GDK_z);
-/*  	accel_button(CODE_UNDO, 0, GDK_BackSpace); */
 
 	accel_button(CODE_REDO, GDK_CONTROL_MASK, GDK_r);
 
@@ -121,6 +116,12 @@ void assign_accel(void)
 
 	accel_button(CODE_EXIT, GDK_CONTROL_MASK,GDK_q);
 
+/* other accelerators, like GDK_Backspace, which cannot be connected 
+ * to buttons, are defined in b_main.c, in the function key_press_handler
+ * One may ask why this function is not in this file. The answer is that
+ * its header mentions GTK objects, and I want not to have such headers
+ * in b_headers.h */
+	
 	gtk_window_add_accel_group(GTK_WINDOW(body->main_window),
 				   body->accel_group);
 }
