@@ -8,8 +8,8 @@
  */
 
 #include <stdio.h>
-#include <math.h>
 #include <stdlib.h>
+#include <math.h>
 
 #include "codes.h"
 #include "const.h"
@@ -84,9 +84,8 @@ void perform_binary_operation(void)
 		break;
 	case CODE_POW  :  
 		if ( !(
-			(cpu->y == 0 && cpu->x <0) /*  || */
-/*  			(cpu->y < 0 && (cpu->x - trunc(cpu->x) != 0)) */
-			/* may be not quite reliable */
+			(cpu->y == 0 && cpu->x <= 0) || 
+  			(cpu->y < 0 && (cpu->x != floor(cpu->x))) 
 			) )
 			cpu->x = pow(cpu->y, cpu->x);
 		else
@@ -153,13 +152,14 @@ void call_exch_xy(void)
 	aftermath();
 }
 
-void call_exch_xz(void)
+void call_stack_up(void)
 {
-	double tmp;
+	double tmp1, tmp2;
+	t_stack_element *stack_element;
 
 	/* this operation is not meant for the algebraic mode */
 	if ( !(cpu->rpn_mode)) {
-		error_occured("call_exch_xz() called in algebraic mode", 
+		error_occured("call_stack_up() called in algebraic mode", 
 			      FALSE);
 		return;
 	}
@@ -167,22 +167,68 @@ void call_exch_xz(void)
 	if (cpu->last_action == ACTION_INPUT)
 		input_to_x();
 
-	tmp = cpu->x;
-	cpu->x = cpu->z;
- 	cpu->z = tmp;
+	if (cpu->stack_mode == STACK_MODE_INFINITE)
+	{
+		if (cpu->stack_head != NULL)
+		{
+
+			tmp1 = cpu->stack_head->z;
+			cpu->stack_head->z = cpu->t;
+			cpu->t = cpu->z;
+			cpu->z = cpu->y;
+			cpu->y = cpu->x;
+
+			stack_element = cpu->stack_head;
+
+			while ( stack_element->next != NULL )
+			{
+				tmp2 = tmp1;
+				tmp1 = stack_element->next->z;
+				stack_element->next->z = tmp2;
+				stack_element = stack_element->next;
+			}
+			cpu->x = tmp1;
+		} else {
+			tmp1 = cpu->x; /* for consistency */
+			if (cpu->y == 0.0 && cpu->z == 0.0 && cpu->t == 0.0)
+				goto done_with_stack_scrolling;
+			tmp1 = cpu->y;
+			cpu->y = cpu->x;
+			if (cpu->z == 0.0 && cpu->t == 0.0)
+				goto done_with_stack_scrolling;
+			tmp2 = tmp1;
+			tmp1 = cpu->z;
+			cpu->z = tmp2;
+			if (cpu->t == 0.0)
+				goto done_with_stack_scrolling;
+			tmp2 = tmp1;
+			tmp1 = cpu->t;
+			cpu->t = tmp2;
+		done_with_stack_scrolling: 
+			cpu->x = tmp1;
+		}
+	} else {
+		tmp1 = cpu->x;
+		cpu->x = cpu->t;
+		cpu->t = cpu->z;
+		cpu->z = cpu->y;
+		cpu->y = tmp1;
+	}
 
 	cpu->last_action = ACTION_ENTER;
 
 	aftermath();
 }
 
-void call_exch_xt(void)
+void call_stack_down(void)
 {
 	double tmp;
+	t_stack_element *stack_element;
 
 	/* this operation is not meant for the algebraic mode */
-	if ( !(cpu->rpn_mode)) {
-		error_occured("call_exch_xt() called in algebraic mode", 
+	if ( !(cpu->rpn_mode)) 
+	{
+		error_occured("call_stack_down() called in algebraic mode", 
 			      FALSE);
 		return;
 	}
@@ -191,8 +237,49 @@ void call_exch_xt(void)
 		input_to_x();
 
 	tmp = cpu->x;
-	cpu->x = cpu->t;
- 	cpu->t = tmp;
+
+	if (cpu->stack_mode == STACK_MODE_INFINITE)
+	{
+		if (cpu->stack_head != NULL)
+		{
+			cpu->x = cpu->y;
+			cpu->y = cpu->z;
+			cpu->z = cpu->t;
+			cpu->t = cpu->stack_head->z;
+
+			stack_element = cpu->stack_head;
+			while ( stack_element->next != NULL )
+			{
+				stack_element->z = stack_element->next->z;
+				stack_element = stack_element->next;
+			}
+			stack_element->z = tmp;
+		} else {
+			if (cpu->y == 0.0 && cpu->z == 0.0 && cpu->t == 0.0)
+				goto done_with_stack_scrolling;
+			cpu->x = cpu->y;
+			if (cpu->z == 0.0 && cpu->t == 0.0)
+			{
+				cpu->y = tmp;
+				goto done_with_stack_scrolling;
+			}
+			cpu->y = cpu->z;
+			if (cpu->t == 0.0)
+			{
+				cpu->z = tmp;
+				goto done_with_stack_scrolling;
+			}
+			cpu->z = cpu->t;
+			cpu->t = tmp;
+		done_with_stack_scrolling: 
+			;
+		}
+	} else {
+		cpu->x = cpu->y;
+		cpu->y = cpu->z;
+		cpu->z = cpu->t;
+		cpu->t = tmp;
+	}
 
 	cpu->last_action = ACTION_ENTER;
 
@@ -202,7 +289,8 @@ void call_exch_xt(void)
 
 void call_eq(void)
 {
-	if (cpu->rpn_mode) {
+	if (cpu->rpn_mode) 
+	{
 		error_occured("call_eq() called in RPN mode", FALSE);
 		return;
 	}
@@ -222,12 +310,14 @@ void call_eq(void)
 void call_enter(void)
 {
 	/* this operation is not meant for the non-RPN mode */
-	if (!(cpu->rpn_mode)) {
+	if (!(cpu->rpn_mode)) 
+	{
 		error_occured("call_enter() called in algebraic mode", FALSE);
 		return;
 	}
 
-	if (cpu->enter_mode == ENTER_MODE_TRADITIONAL) {
+	if (cpu->enter_mode == ENTER_MODE_TRADITIONAL) 
+	{
 		if (cpu->last_action == ACTION_INPUT)
 			input_to_x();
 		push_stack();
@@ -252,12 +342,14 @@ void call_enter(void)
 
 void call_left_paren(void)
 {
-	if (cpu->rpn_mode) {
+	if (cpu->rpn_mode) 
+	{
 		error_occured("call_left_paren() called in RPN mode", FALSE);
 		return;
 	}
 
-	if (cpu->last_action == ACTION_BINOP) {
+	if (cpu->last_action == ACTION_BINOP) 
+	{
 		cpu->number_of_parens++;
 		aftermath();
 	}
@@ -265,16 +357,19 @@ void call_left_paren(void)
 
 void call_right_paren(void)
 {
-	if (cpu->rpn_mode) {
+	if (cpu->rpn_mode) 
+	{
 		error_occured("call_right_paren() called in RPN mode", FALSE);
 		return;
 	}
 
- 	if (cpu->last_action == ACTION_INPUT) {
+ 	if (cpu->last_action == ACTION_INPUT) 
+	{
 		input_to_x();
 	}
 
-	if (!no_open_parens()) {
+	if (!no_open_parens()) 
+	{
 		if (cpu->op != CODE_NO_OPERATION) {
 			perform_binop_chain(PRIORITY_CODE_MIN, TRUE);
 		}
@@ -302,15 +397,18 @@ void push_stack(void)
 {
 	t_stack_element *new_element;
 
-	if (cpu->rpn_mode) {
-		if (cpu->stack_mode == STACK_MODE_INFINITE ) {
-			new_element = (t_stack_element*) 
-				malloc (sizeof(t_stack_element));
-			if(new_element == NULL)
-				error_occured("malloc failed", TRUE);
-			new_element->next = cpu->stack_head;
-			new_element->z = cpu->t;
-			cpu->stack_head = new_element;
+	if (cpu->rpn_mode) 
+	{
+		if ( (cpu->stack_mode == STACK_MODE_INFINITE)
+		     && (cpu->stack_head != NULL || cpu->t != 0.0 ))
+		{
+				new_element = (t_stack_element*) 
+					malloc (sizeof(t_stack_element));
+				if(new_element == NULL)
+					error_occured("malloc failed", TRUE);
+				new_element->next = cpu->stack_head;
+				new_element->z = cpu->t;
+				cpu->stack_head = new_element;
 		}
 		cpu->t = cpu->z;
 		cpu->z = cpu->y;
@@ -333,11 +431,14 @@ void pop_stack(void)
 {
 	t_stack_element *to_remove;
 	
-	if (cpu->rpn_mode) {
+	if (cpu->rpn_mode) 
+	{
 		cpu->y = cpu->z;
 		cpu->z = cpu->t;
-		if (cpu->stack_mode == STACK_MODE_INFINITE ) {
-			if (cpu->stack_head != NULL) {
+		if (cpu->stack_mode == STACK_MODE_INFINITE ) 
+		{
+			if (cpu->stack_head != NULL) 
+			{
 				to_remove = cpu->stack_head;
 				cpu->t = (cpu->stack_head)->z;
 				cpu->stack_head = cpu->stack_head->next;
@@ -347,7 +448,8 @@ void pop_stack(void)
 			}
 		}
 	} else {
-		if (cpu->stack_head != NULL) {
+		if (cpu->stack_head != NULL) 
+		{
 			to_remove = cpu->stack_head;
 			cpu->y = (cpu->stack_head)->z;
 			cpu->op = (cpu->stack_head)->op;
@@ -373,7 +475,8 @@ int no_open_parens(void)
 
 	current = cpu->stack_head;
 
-	while ( current != NULL ) {
+	while ( current != NULL ) 
+	{
 		if (current->number_of_parens > 0)
 			return FALSE;
 		current = current->next;
@@ -435,7 +538,7 @@ void call_unary_op(int received_code)
 			cpu->x_overflow = TRUE;
 		break;
 	case CODE_SIN:
-		if (cpu->angle_measure == CODE_DEG)
+		if (cpu->angle_units == CODE_DEG)
 			cpu->x *= PI / 180.0;
 		if (almost_integer(cpu->x / PI, cpu->precision))
 			/* hack to have sin 180 == 0 */ 
@@ -446,7 +549,7 @@ void call_unary_op(int received_code)
 	case CODE_ASIN:
 		if (fabs(cpu->x) <= 1.0) {
 			cpu->x = asin(cpu->x);
-			if (cpu->angle_measure == CODE_DEG)
+			if (cpu->angle_units == CODE_DEG)
 				cpu->x *= 180.0 / PI;
 		}
 		else
@@ -459,7 +562,7 @@ void call_unary_op(int received_code)
 		cpu->x = asinh (cpu->x);
 		break;
 	case CODE_COS :
-		if (cpu->angle_measure == CODE_DEG)
+		if (cpu->angle_units == CODE_DEG)
 			cpu->x *= PI / 180.0;
 		if (almost_integer((cpu->x - PI / 2.0) / PI, 
 				   cpu->precision)
@@ -473,7 +576,7 @@ void call_unary_op(int received_code)
 	case CODE_ACOS:
 		if (fabs(cpu->x) <= 1.0) {
 			cpu->x = acos(cpu->x);
-			if (cpu->angle_measure == CODE_DEG)
+			if (cpu->angle_units == CODE_DEG)
 				cpu->x *= 180.0 / PI;
 		}
 		else
@@ -489,7 +592,7 @@ void call_unary_op(int received_code)
 			cpu->x_overflow = TRUE;
 		break;
 	case CODE_TAN:
-		if (cpu->angle_measure == CODE_DEG)
+		if (cpu->angle_units == CODE_DEG)
 			cpu->x *= PI / 180.0;
 		if (almost_integer((cpu->x - PI / 2.0) / PI, 
 				   cpu->precision)
@@ -503,7 +606,7 @@ void call_unary_op(int received_code)
 		break;
 	case CODE_ATAN:
 		cpu->x = atan(cpu->x);
-		if (cpu->angle_measure == CODE_DEG)
+		if (cpu->angle_units == CODE_DEG)
 			cpu->x *= 180.0 / PI;
 		break;
 	case CODE_TANH :
