@@ -86,6 +86,8 @@ void cpu_to_output(void)
 			cpu->op_d[2]->show_brace = TRUE;
 
 	done_with_reg_displays: 
+		; /* this is to prevent compiler warnings about
+		   * "deprecated label at the end of a compound statement" */
 	}
 	
 	for (i = 0; i < NUMBER_OF_REG_DISPLAYS; i++) {
@@ -104,6 +106,7 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 	int i;
 	int signif_digit_num, head_zeros_num;
 	int exp_eng_corr;
+	int output_length;
 
 	/* In principle, this function could contain just simple processing
 	 * of the output of sprintf("%e",x). However, I have chosen to write
@@ -111,7 +114,7 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 	 * the resulting function would probably be approximately the same 
 	 * anyway */
 
-	/* First, we consider some special cases of too large or too small x */
+/* First, we consider some special cases of too large or too small x */
 
 	if (a_overflow) {
 		display->display_overflow = TRUE;
@@ -121,7 +124,7 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 
 	abs_x = fabs(a);
 
-	if (abs_x >= pow(10, pow(10,EXP_INPUT_LENGTH))) {
+	if (abs_x >= pow(10, pow(10, EXP_INPUT_LENGTH))) {
 	/* x is so large that it cannot be displayed */
 		display->display_overflow = TRUE;
 		return;
@@ -130,9 +133,9 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 
 	display->display_overflow = FALSE;
 	
-	if (abs_x < pow(10, -(pow(10,EXP_INPUT_LENGTH)-1))) {
+	if (abs_x < pow(10, -(pow(10, EXP_INPUT_LENGTH)-1))) {
 	/* x is effectively zero */
-		display->format = FORMAT_FP;
+		display->format = FORMAT_FIX;
 		display->n_int = 1;
 		display->int_field[0] = CODE_0;
 		display->n_frac = 0;
@@ -141,10 +144,10 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 	}
 	/* x is not zero, continue */
 
-	/* Now, the special cases have been considered, and we can proceed
-	 * with a regular case. This is not trivial, however. The main problem
-	 * arises when x=0.9999999999999...., so rounding of x for displaying 
-	 * may lead to changes in the exp part, see the "!!!" comment */
+/* Now, the special cases have been considered, and we can proceed
+ * with a regular case. This is not trivial, however. The main problem
+ * arises when x=0.9999999999999...., so rounding of x for displaying 
+ * may lead to changes in the exp part, see the "!!!" comment */
 
 	/* sign */
 	if (a >= 0)
@@ -154,11 +157,20 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 	
 	intlog10_x = round(floor(log10(abs_x)));
 
-        /* truncating all but first "INPUT_LENGTH" digits of x */
-	abs1_x = rint((abs_x / pow(10, intlog10_x + 1)) 
-		      * pow(10,INPUT_LENGTH));
+	if (cpu->rounding)
+		output_length = cpu->digits_to_keep;
+	else
+		output_length = INPUT_LENGTH;
 
-	if (abs1_x >= pow(10,INPUT_LENGTH)) {
+        /* truncating all but first "output_length" digits of x */
+	/* for numbers <1 that can be displayed without exponent this
+	 * interpretation will be redone later, becuase in this case 
+	 * additional rounding may be needed */
+
+	abs1_x = rint((abs_x / pow(10, intlog10_x + 1)) 
+		      * pow(10, output_length));
+
+	if (abs1_x >= pow(10, output_length)) {
 	/* !!! 
 	 * rounding has promoted x to the next order - 
 	 * this is what we mentioned before */
@@ -167,12 +179,13 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 	}
 
 	if ( (intlog10_x == -1) 
-	     && (rint(abs1_x/10) >= pow(10,INPUT_LENGTH - 1)) ) {
+	     && (cpu->prescribed_format == FORMAT_FIX)
+	     && (rint(abs1_x/10) >= pow(10,output_length - 1)) ) {
 		/* a very special case of, say,  x=0.999999997, when the 
 		 * introduction of the zero before the dot changes the integer 
 		 * part from 0 to 1 thus leading to switching between the 
 		 * types of format distinguished  below */
-		display->format = FORMAT_FP;
+		display->format = FORMAT_FIX;
 		display->n_int = 1;
 		display->int_field[0] = CODE_1;
 		display->n_frac = 0;
@@ -180,21 +193,24 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 		return;
 	}
 
+	/* padding with zeros to the full INPUT_LENGTH */
+	abs1_x *= pow(10, INPUT_LENGTH - output_length);
+
 	/* interpreting meaningful digits of x */
-	for (i=INPUT_LENGTH-1; i >= 0; i--) {
+	for (i = INPUT_LENGTH - 1; i >= 0; i--) {
 		digits_x[i] = digit_to_code(last_digit(abs1_x));
-		abs1_x = rint( (abs1_x-last_digit(abs1_x)) / 10 );
+		abs1_x = rint( (abs1_x - last_digit(abs1_x)) / 10 );
 	}
 	
-	/* By this point, we have determined all the digits of x to display
-	 * and the exponent. Now we are going to figure out what digits to
-	 * put before and after the dot */
+/* By this point, we have determined all the digits of x to display
+ * and the exponent. Now we are going to figure out what digits to
+ * put before and after the dot */
 
 	if ( (0 <= intlog10_x) && (intlog10_x < INPUT_LENGTH) 
-	     && (cpu->prescribed_format == FORMAT_FP) ) { 
+	     && (cpu->prescribed_format == FORMAT_FIX) ) { 
 	/* x can be displayed without the exponent 
 	 * and x>=1, so there is NO need to introduce heading zeros */
-		display->format = FORMAT_FP;
+		display->format = FORMAT_FIX;
 
 		display->n_int = intlog10_x + 1;
 		for (i = 0 ; i < display->n_int; i++) 
@@ -205,10 +221,10 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 			display->frac_field[i] = 
 				digits_x[display->n_int + i];
 	} else if ( (-INPUT_LENGTH < intlog10_x) && (intlog10_x < 0) 
-		   && (cpu->prescribed_format == FORMAT_FP) ) {
+		   && (cpu->prescribed_format == FORMAT_FIX) ) {
 	/* x can be displayed without the exponent 
 	 * and x<1, so there IS need to introduce heading zeros */
-		display->format = FORMAT_FP;
+		display->format = FORMAT_FIX;
 
 		display->n_int = 1;
 		display->int_field[0] = CODE_0;
@@ -219,27 +235,45 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 			display->frac_field[i] = CODE_0;
 
 		signif_digit_num = INPUT_LENGTH - 1 - head_zeros_num;
+		if (signif_digit_num > output_length)
+			signif_digit_num = output_length;
 
 		/* separating first "signif_digit_num" digits of x */
 		abs2_x = rint ( (abs_x / pow(10, intlog10_x+1)) 
 				* pow(10, signif_digit_num) );
 		
-		if (abs2_x >= pow(10,signif_digit_num)) 
+		if (abs2_x >= pow(10,signif_digit_num)) {
 			/* rounding promoted x to the next order */
-			display->frac_field[head_zeros_num - 1] = CODE_1;
+			if (signif_digit_num < output_length){
+				signif_digit_num++;
+				head_zeros_num--;
+			}
+			else 
+				abs2_x = rint(abs2_x / 10);
+		}
 
-		/* interpreting meaningful digits of x */
-		for(i = (INPUT_LENGTH - 1) - 1; i>=head_zeros_num ; i--){
-			display->frac_field[i] = 
-				digit_to_code( last_digit(abs2_x) );
+
+		for (i = signif_digit_num - 1; i >= 0; i--) {
+			digits_x[i] = digit_to_code(last_digit(abs2_x));
 			abs2_x = rint( (abs2_x-last_digit(abs2_x)) / 10 );
 		}
+
+		/* interpreting meaningful digits of x */
+		for(i = 0; i < signif_digit_num; i++){
+			display->frac_field[i + head_zeros_num] = digits_x[i];
+		}
+
+		/* padding the rest wth zeros */
+		for(i = head_zeros_num + signif_digit_num; i < INPUT_LENGTH; 
+		    i++)
+			display->frac_field[i] = CODE_0;
+
 	} else {
 	/* exponent is needed to display x */
-		display->format = FORMAT_EXP;
+		display->format = FORMAT_SCI;
 
-		if (cpu->prescribed_format == FORMAT_EXP || 
-		    cpu->prescribed_format == FORMAT_FP) {
+		if (cpu->prescribed_format == FORMAT_SCI || 
+		    cpu->prescribed_format == FORMAT_FIX) {
 			display->n_int = 1;
 			display->int_field[0] = digits_x[0];
 
@@ -279,13 +313,17 @@ void a_to_display(double a, t_cpu_display *display, int a_overflow)
 			else 
 				display->exp_sign = SIGN_MINUS;
 		} else
-			display->format = FORMAT_FP;
+			display->format = FORMAT_FIX;
 			
 	}
 
 	/* getting rid of trailing zeros in frac */
 	while ((display->frac_field[display->n_frac-1] == CODE_0)
-	       && (display->n_frac > 0) ) 
+	       && (display->n_frac > 0) 
+	       && !( (cpu->rounding)
+		     && (display->n_frac + display->n_int <= output_length)
+		     && (!cpu->trunc_zeros)
+		       )) 
 		display->n_frac--;
 	
 }
