@@ -1,7 +1,7 @@
 /*
  * Calcoo: gtkaux.c
  *
- * Copyright (C) 2001 Alexei Kaminski
+ * Copyright (C) 2001, 2002 Alexei Kaminski
  *
  * auxiliary functions for operations with gtk widgets
  * no functions related to specific calcoo tasks
@@ -17,11 +17,14 @@
 
 #include "codes.h"
 #include "body.h"
+#include "defaults.h"
 #include "gtkaux_headers.h"
+#include "aux_headers.h"
 #include "io_headers.h" 
 
 void create_fg_pixmap(GtkWidget **pixmap_widget, char **pixmap_xpm)
-/* creates a monochrome gtk pixmap having the color of the gtk foreground */
+/* creates a monochrome gtk pixmap having the color of the gtk foreground 
+ * out of the given pixmap */
 {
 	GdkPixmap *pixmap, *all_black;
 	GdkBitmap *mask;
@@ -57,16 +60,19 @@ void create_fg_pixmap(GtkWidget **pixmap_widget, char **pixmap_xpm)
 	/* creating a bitmap with all pixels having the color of foreground, 
 	 * of the size of the original pixmap */
 	all_black = gdk_pixmap_create_from_data(
-		                   (body->main_window)->window,
-				   black_bits,
-				   w, h,
-				   -1,
-				   &(body->style)->fg[GTK_STATE_NORMAL],
-				   &(body->style)->bg[GTK_STATE_NORMAL]);
+		(body->main_window)->window,
+		black_bits,
+		w, h,
+		-1,
+		&(body->style)->fg[GTK_STATE_FOR_CALCOO_FOREGROUND_COLOR],
+		&(body->style)->bg[GTK_STATE_NORMAL]);
 	
-	/* imposing the transparency mask of the original pixmap
-	 * onto the foreground rectangle */
-	*pixmap_widget = gtk_pixmap_new(all_black, mask);
+	if (CONVERT_ICONS_TO_FOREGROUND_COLOR)
+		/* imposing the transparency mask of the original pixmap
+		 * onto the foreground rectangle */
+		*pixmap_widget = gtk_pixmap_new(all_black, mask);
+	else
+		*pixmap_widget = gtk_pixmap_new(pixmap, mask);
 
 	free(black_bits);
 	gdk_pixmap_unref(pixmap);
@@ -113,7 +119,12 @@ void get_selection(GtkWidget *widget, GtkSelectionData *data, gpointer val)
 	gchar *buff;
 	int success;
 	double x;
-	
+
+	if(data->length < 0) /* the selection could not be retrieved */
+	{
+		mess("the selection could not be retrieved", 0);
+		return;
+	}
 	buff = strcat(data->data,"\n");
 	success = sscanf(buff,"%lf", &x);
 	if (success)
